' //////////////////////////////////////////////////////////////////////
' Hydra Logo
' 
' AUTHOR: Joshua Meeds
' LAST MODIFIED: 3-30-2007
'
' Parts based on Keyboard Demo by Andre' LaMothe
'
' //////////////////////////////////////////////////////////////////////

'///////////////////////////////////////////////////////////////////////
' CONSTANTS SECTION ////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

CON

  _clkmode = xtal1 + pll8x              ' enable external clock and pll times 8
  _xinfreq = 10_000_000 + 0000          ' set frequency to 10 MHZ plus some error
  _stack = ($2A00 + $100) >> 2          ' accomodate display memory and stack

   ' graphics driver and screen constants
  PARAMCOUNT        = 14
  
   ' size of graphics tile map
  X_TILES           = 12
  Y_TILES           = 14

  SCREEN_WIDTH      = 192
  SCREEN_HEIGHT     = 216
  TEXTCHARS         = 32
  TEXTLINEHEIGHT    = 11

  LONGBYTES         = 4
  LONGSPERCOL       = 224

  TITLE_Y_NOOVERSCAN  = 44
  TITLE_Y_OVERSCAN  = 28

  KEYCODE_ESC       = $CB
  KEYCODE_ENTER     = $0D
  KEYCODE_BACKSPACE = $C8
  KEYCODE_SPACE     = $20

  NO_NEW_LINE       = $FF
  CHARS_PER_LINE    = 32

  ONSCREEN_BUFFER   = $5600             ' onscreen buffer
  SCREEN_BUFFER_LONGS = 2688
  
  SIN_TABLE         = $E000
  SIN_TABLE_END     = $F000
  NATIVE_ANGLE_90   = $800
  NATIVE_ANGLE_180  = $1000

  NOTES_C2          = $3000
  NOTES_D2          = $3680
  NOTES_E2          = $3D00
  NOTES_F2          = $4200
  NOTES_G2          = $4880
  NOTES_A2          = $5180
  NOTES_B2          = $5C00
  NOTES_C3          = $6000
  NOTES_REST        = 0

  ' NES bit encodings for NES gamepad 0
  NES0_RIGHT  = %00000000_00000001
  NES0_LEFT   = %00000000_00000010
  NES0_DOWN   = %00000000_00000100
  NES0_UP     = %00000000_00001000
  NES0_START  = %00000000_00010000
  NES0_SELECT = %00000000_00100000
  NES0_B      = %00000000_01000000
  NES0_A      = %00000000_10000000

  ' NES bit encodings for NES gamepad 1
  NES1_RIGHT  = %00000001_00000000
  NES1_LEFT   = %00000010_00000000
  NES1_DOWN   = %00000100_00000000
  NES1_UP     = %00001000_00000000
  NES1_START  = %00010000_00000000
  NES1_SELECT = %00100000_00000000
  NES1_B      = %01000000_00000000
  NES1_A      = %10000000_00000000

  ERROR_NONE        = 0
  ERROR_NORMAL      = 1
  ERROR_REPEAT      = 2

  START_NEW_FUNCION_NOW = 2

  COMMAND_STACKSIZE = 16
  COMMAND_MAXLINES  = 15
  COMMAND_MAXNUM    = 10
  PAGES_PER_FUNCTION = 2
  COMMAND_TOC_PAGE = 128
  FUNCTION_START_PAGE = 129
  ENDTOC_START = 250

  VARIABLE_MAXCHARS = 8
  VARIABLES_MAX = 8
  VARIABLES_LOCALMAX = 4

  COMMAND_FD       = 0
  COMMAND_FORWARD  = 1
  COMMAND_BK       = 2
  COMMAND_BACKWARD = 3
  COMMAND_LT       = 4
  COMMAND_LEFT     = 5
  COMMAND_RT       = 6
  COMMAND_RIGHT    = 7
  COMMAND_HOME     = 8
  COMMAND_SETPOS   = 9
  COMMAND_HIDETURTLE = 10
  COMMAND_HT       = 11
  COMMAND_SHOWTURTLE = 12
  COMMAND_ST       = 13
  COMMAND_PENUP    = 14
  COMMAND_PU       = 15
  COMMAND_PENDOWN  = 16
  COMMAND_PD       = 17
  COMMAND_REPEAT   = 18
  COMMAND_RP       = 19
  COMMAND_TO       = 20
  COMMAND_MAKE     = 21
  COMMAND_MK       = 22
  COMMAND_SHOOT    = 23
  COMMAND_RESET    = 24
  COMMAND_PLAYSONG = 25
  COMMAND_SAVE     = 26
  COMMAND_CLEARSAVE = 27
  COMMAND_IF       = 28
  COMMAND_BLOCKOPEN = 29
  COMMAND_BLOCKCLOSE= 30
  COMMAND_PRINT    = 31
  COMMAND_STOP     = 32
  COMMAND_SETTURBO = 33
  COMMAND_NOOVERSCAN = 34
  COMMAND_OVERSCAN = 35
  COMMAND_IFPAD1   = 36
  COMMAND_IFPAD2   = 37
  COMMAND_END      = 38
  ' COMMAND_END *MUST* be last

  'Total number of command bytes (name string chars)
  INTERNAL_COMMAND_LENGTH = 183
  
'///////////////////////////////////////////////////////////////////////
' VARIABLES SECTION ////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

VAR

  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles] ' storage for screen tile map
  long  colors[64]                ' color look up table

  long  screenbackup[3 * CHARS_PER_LINE]
  
  ' string/key stuff
  byte sbuffer[2]
  byte screentext[CHARS_PER_LINE * 5]
  byte linelength[5]
  byte curr_key
  byte temp_key 
  long data
  byte newLine
  byte commandPos
  byte secondaryReturn
  byte userMessage[26]
  byte genericError
  byte textbackup[CHARS_PER_LINE]

  ' User-made command stuff (functions)
  byte newcommand[COMMAND_MAXNUM]
  byte newfunctionlocalvars[COMMAND_STACKSIZE * VARIABLES_LOCALMAX * VARIABLE_MAXCHARS]
  byte newfunctionLocalVarNum[COMMAND_STACKSIZE * VARIABLES_LOCALMAX]
  byte newfunction[CHARS_PER_LINE * COMMAND_MAXLINES]
  byte functionLine[COMMAND_STACKSIZE]
  byte functionPos[COMMAND_STACKSIZE]
  byte functionBackupLine[COMMAND_STACKSIZE * CHARS_PER_LINE]
  byte functionBackupLinelength[COMMAND_STACKSIZE]
  byte functionBackupCommandPos[COMMAND_STACKSIZE]
  byte functionParameterList[COMMAND_STACKSIZE]
  byte ifresult[COMMAND_STACKSIZE]
  byte ifblock[COMMAND_STACKSIZE]
  byte commandStackPos
  byte toMode
  byte backupcommandlength
  byte commandFinalLine[COMMAND_STACKSIZE]
  byte endOfFunction[COMMAND_STACKSIZE]

  byte EeepromBuffer[32]
  byte turbo_page

  '  Variables
  long localVariableStorage[VARIABLES_LOCALMAX * COMMAND_STACKSIZE]
  
  ' terminal, turtle vars
  byte row, column
  long cursorx, cursory
  long turtlex, turtley
  long turtleangle
  byte turtlevisible
  byte pendown
  byte inputDisabled
  byte EscapeHit
  
  byte COMMAND_MAX
  long COMMAND_LENGTH

  byte TITLE_Y
  byte TEXT_START_Y


  long soundFreq
  long soundLength
  long songPos
    
  ' counter vars
  long curr_cnt, end_cnt

  long inputstack[64]
  long interpretstack[1024]
  long soundstack[64]

'///////////////////////////////////////////////////////////////////////
' OBJECT DECLARATION SECTION ///////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////
OBJ

  tv    : "tv_drv_010.spin"                        ' instantiate a tv object
  gr    : "graphics_drv_010.spin"                  ' instantiate a graphics object
  key   : "keyboard_iso_010.spin"                  ' instantiate a keyboard object
  eeprom: "JMM_eeprom_011.spin"                    ' instantiate an eeprom object
  controller: "gamepad_drv_001.spin"               ' instantiate an controller object

'///////////////////////////////////////////////////////////////////////
' EXPORT PUBLICS  //////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

PUB start  | i, dx, dy, TOCaddress

  'start keyboard on pingroup 3 
  key.start(3)
  
  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

  'start and setup graphics
  gr.start
  gr.setup(X_TILES, Y_TILES, SCREEN_WIDTH/2, SCREEN_HEIGHT/2, onscreen_buffer)
    
  'init colors
  repeat i from 0 to 64
    colors[i] := $00001010 * (i+4) & $F + $FB060C02

  'init tile screen
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := onscreen_buffer >> 6 + dy + dx * tv_vc + ((dy & $3F) << 10)

  ' start controller COG
  controller.start

  TITLE_Y := TITLE_Y_OVERSCAN
  TEXT_START_Y := TITLE_Y + 16
      
  ' write title
  gr.textmode(1,1,5,0)
  gr.colorwidth(2,0)
  gr.text(-SCREEN_WIDTH/2, -TITLE_Y, @title_string)
  gr.colorwidth(3,0)
  gr.plot(-192/2, -TITLE_Y)
  gr.line(192/2,  -TITLE_Y)

  'draw the turle, text cursor
  turtlex := 0
  turtley := 30
  turtleangle := 0
  gr.colorwidth(1,0)
  gr.vec(turtlex, turtley, $100, turtleangle, @turtlesprite)
  DrawCursor(-SCREEN_WIDTH/2 + column*6, -TEXT_START_Y - row*TEXTLINEHEIGHT + 4)

  newLine := NO_NEW_LINE
  commandPos := 0
  commandStackPos := 0
  turtlevisible := 1
  pendown := 1
  toMode := 0
  inputDisabled := 0
  genericError := ERROR_NONE
  EscapeHit := 0
  COMMAND_MAX := COMMAND_END
  COMMAND_LENGTH := INTERNAL_COMMAND_LENGTH
  eeprom.SetTurboPage(FUNCTION_START_PAGE,0)
  turbo_page := 0                            ' the user-set turbo_page is 0

  'start the COG's that run everything
  cognew(inputLoop, @inputstack)
  cognew(interpretLoop, @interpretstack)
  cognew(soundLoop, @soundstack)

 ' BEGIN LOOP ////////////////////////////////////////////////////
  repeat
        
' ////////////////////////////////////////////////////////////////////

'///////////////////////////////////////////////////////////////////////
' LOCAL PRIVATES  //////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////
 
''---------------------------------------------------------------------
''soundLoop
''Purpose: COG, handles all sound playing
''Params: none
''Returns:nothing
''---------------------------------------------------------------------
PRI soundLoop | sd_freq
' Initialize registers
  CTRA := %0_00100_000_00000_000000000_000000111
  FRQA := $0
  DIRA := %00000000_00000000_00000000_10000000

  songPos := 0         ' Set this to 1 to play song

  repeat
    if (songPos > 0)
      soundFreq := songs[songPos-1]
      if (soundFreq == 0)
        FRQA := 0
        songPos := 0
      else
        songPos++
        FRQA := soundFreq
        soundLength := 8192*songs[songPos-1] 
        songPos++
        repeat while (soundLength > 0)
          soundLength--
' soundLoop
  
''---------------------------------------------------------------------
''inputLoop
''Purpose: COG, handles all input, drawing text and storing command buffer
''Params: none
''Returns:nothing
''---------------------------------------------------------------------
PRI inputLoop 
  ' initialize terminal cursor position
  column :=0
  row := 0

 ' BEGIN LOOP
  repeat 
    ' get key    
    if (key.gotkey==TRUE)
      curr_key := key.getkey
      if (inputDisabled == 0)
        'print character to screen
        Print_To_Term(curr_key)
      elseif (curr_key == KEYCODE_ESC)
        EscapeHit := 1
    else ' gotkey 
      curr_key := 0
' inputLoop      

''---------------------------------------------------------------------
''interpretLoop
''Purpose: COG, takes stored lines of text and interprets them 
''Params: none
''Returns:nothing
''---------------------------------------------------------------------
PRI interpretLoop | PageNumber
  inputDisabled := 1
  LoadFromEeprom
  inputDisabled := 0
 
  repeat
    ifnot (newLine == NO_NEW_LINE)
    ' User has entered a line
      if (toMode == 0)
      ' ' run command
        inputDisabled := 1
        commandPos := 0
        interpretCommand
        inputDisabled := 0
        EscapeHit := 0

      ' See if we need to display an error message
        if (genericError == ERROR_NORMAL)
          Print_Message(@huhmessage)
          newLine := NO_NEW_LINE
          genericError := ERROR_NONE
        elseif (genericError == ERROR_REPEAT)
          Print_Message(@huhrepeatmessage)
          newLine := NO_NEW_LINE
          genericError := ERROR_NONE
        
      else
      ' ' store command
        functionPos[0] := 0
        repeat while(screentext[newLine*TEXTCHARS + functionPos[0]] <> KEYCODE_ENTER)                                                                                                          
        ' Copy this line into the function
          newfunction[functionLine[0]*TEXTCHARS + functionPos[0]] := screentext[newLine*TEXTCHARS + functionPos[0]]          
          functionPos[0]++
        newfunction[functionLine[0]*TEXTCHARS + functionPos[0]] := KEYCODE_ENTER

      ' Write the function to EEPROM
        if ((screentext[newLine*TEXTCHARS] == "e") and (screentext[newLine*TEXTCHARS + 1] == "n") and (screentext[newLine*TEXTCHARS + 2] == "d") and (screentext[newLine*TEXTCHARS + 3] == KEYCODE_ENTER))
          PageNumber := (((COMMAND_MAX - 1) - COMMAND_END)*PAGES_PER_FUNCTION) + FUNCTION_START_PAGE
          if (turbo_page == 0)
            eeprom.SetTurboPage(PageNumber,0)     'If no user-set turbo page, the most recent command is turboed 
          newfunction[0] := functionLine[0] + 1    ' store the number of lines in the command as very first byte
          eeprom.WritePages(@newfunction, PageNumber, PAGES_PER_FUNCTION)
          toMode := 0  ' This is the "end" of the function, store it and stop storing more commands

     '   Cancel "to mode", don't save the new command
        elseif ((screentext[newLine*TEXTCHARS] == "c") and (screentext[newLine*TEXTCHARS + 1] == "a") and (screentext[newLine*TEXTCHARS + 2] == "n") and (screentext[newLine*TEXTCHARS + 3] == "c") and (screentext[newLine*TEXTCHARS + 4] == "e") and (screentext[newLine*TEXTCHARS + 5] == "l") and (screentext[newLine*TEXTCHARS + 6] == KEYCODE_ENTER)) 
          toMode := 0  ' Cancel to mode
          COMMAND_MAX--
          COMMAND_LENGTH -= backupcommandlength
          Print_Message(@cancelledmessage)
        elseif (screentext[newLine*TEXTCHARS] == KEYCODE_ESC)
          toMode := 0  ' Cancel to mode
          COMMAND_MAX--
          COMMAND_LENGTH -= backupcommandlength
          Print_Message(@cancelledmessage)
        elseif (functionLine[0] + 1 > COMMAND_MAXLINES)
          toMode := 0  ' Cancel to mode
          COMMAND_MAX--
          COMMAND_LENGTH -= backupcommandlength
          Print_Message(@toomanylinesmessage)

        functionLine[0]++
        newLine := NO_NEW_LINE

' interpretLoop      

''---------------------------------------------------------------------
''interpretCommand
''Purpose: interprets and runs a single command from direct entry or repeat command
''Params: nothing
''Returns:NewStringPosition - the character following the command                
''---------------------------------------------------------------------
PRI interpretCommand : NewStringPosition | Command, StringStart, StringChar, Loop, Huh
' search through known commands
  NewStringPosition := commandPos
  StringStart := 0
  Huh := 1

  repeat Command from 0 to COMMAND_MAX
    StringChar := StringStart+1
    Loop := 0
    repeat while(screentext[newLine*TEXTCHARS + Loop + commandPos] == commands[StringChar])
    ' Check all letters of this command with the entered text
      StringChar++
      Loop++

    if (EscapeHit == 1)
      ' Cancel everything!
      newLine := NO_NEW_LINE
      Command := COMMAND_MAX
      return      

    if ((Loop == commands[StringStart]) and ((screentext[newLine*TEXTCHARS + Loop + commandPos] == " ") or (screentext[newLine*TEXTCHARS + Loop + commandPos] == "]") or (screentext[newLine*TEXTCHARS + Loop + commandPos] == KEYCODE_ENTER))) 
    ' A match! We made it through all the command letters, so it's a valid command
      Huh := 0
      if ((endOfFunction[commandStackPos] == 1) and (Command > COMMAND_END))
        endOfFunction[commandStackPos] := Command         ' Save stack space by joining new command to old
        return (newLine*TEXTCHARS + Loop + commandPos + 1)
      else
        if (endOfFunction[commandStackPos] == 1)
          endOfFunction[commandStackPos] := 0 
        NewStringPosition := runCommand(Command, newLine*TEXTCHARS + Loop + commandPos + 1)
        newLine := NO_NEW_LINE
        Command := COMMAND_MAX
        QUIT
    else
    ' Advance to next command
      StringStart := StringStart + commands[StringStart] + 1

  if (Huh == 1)
    genericError := Huh
    Loop := 0
    repeat while ((screentext[newLine*TEXTCHARS + Loop + commandPos] <> "]") and (screentext[newLine*TEXTCHARS + Loop + commandPos] <> " ")) 
      Loop++
    NewStringPosition := newLine*TEXTCHARS + Loop + commandPos
    
  return(NewStringPosition)
' interpretCommand

''---------------------------------------------------------------------
''Print_To_Term
''Purpose: Prints a text character to the screen at current column, row
''Params: char_code   ASCII Code to draw
''Returns:nothing                
''---------------------------------------------------------------------
PRI Print_To_Term(char_code)
' test for new line
  if (char_code == KEYCODE_ENTER)
    if (linelength[row] == 0)
      return   ' Enter on an empty line?  Ignore it!
    screentext[row*TEXTCHARS + column] := KEYCODE_ENTER
    column :=0
    if (++row > 4)    
      row := 4
      ScrollTextUp
      DrawCursor(-SCREEN_WIDTH/2, cursory)
    else
      DrawCursor(-SCREEN_WIDTH/2, cursory-TEXTLINEHEIGHT)
    linelength[row] := 0
    newLine := row - 1 ' Store newly entered line for interpreter 
' test for backspace (delete)
  elseif (char_code == KEYCODE_BACKSPACE)
    screentext[row*TEXTCHARS + column] := KEYCODE_SPACE
    if (column > 0)
      --column
      --linelength[row]
      DrawCursor(cursorx-6, cursory)
'    else
'   Backup to the previous line (not wanted here, maybe later in edit mode)
'      linelength[row] := 0            
'      --row
'      --linelength[row]
'      column := linelength[row]
'      DrawCursor(-SCREEN_WIDTH/2 + column*6, cursory+TEXTLINEHEIGHT)
' Draw character, advance cursor
  else
    DrawCursor(cursorx+6, cursory)
    ' set the printing buffer up 
    sbuffer[0] := char_code
    sbuffer[1] := 0
    gr.text(-SCREEN_WIDTH/2 + column*6,-TEXT_START_Y - row*TEXTLINEHEIGHT, @sbuffer)
    screentext[row*TEXTCHARS + column] := char_code
    ++linelength[row]

    ' test for text wrapping and new line
    if (++column > TEXTCHARS-1)
      column := 0
      if (++row > 4)    
        row := 4
        ScrollTextUp
        DrawCursor(-SCREEN_WIDTH/2, cursory)
      else
        DrawCursor(-SCREEN_WIDTH/2, cursory-TEXTLINEHEIGHT)
      linelength[row] := 0
        ' scroll text window
  return
' Print_To_Term

''---------------------------------------------------------------------
''Print_Message
''Purpose: Prints a line of text to the screen at current column, row
''Params: message  Address to the message to display.  Automatically adds carriage return
''Returns:nothing                
''---------------------------------------------------------------------
PRI Print_Message(message) : counter
  counter := 0
  repeat while((counter < 30) and (byte[message][counter] > 0))
    Print_To_Term(byte[message][counter])
    counter++
  Print_To_Term(KEYCODE_ENTER)
  return
' Print_Message

''---------------------------------------------------------------------
''DrawCursor
''Purpose: Clear the cursor's current pos, move the cursor to a new pos
''Params: NewPosX     X Position to move the cursor to                                      
''        NewPosY     Y Position to move the cursor to
''Returns:nothing
''---------------------------------------------------------------------
PRI DrawCursor(NewPosX, NewPosy)
  'First, clear the old cursor (as long as it's been drawn before)
  ifnot (cursory == 0)
    gr.color(0)
    gr.quad(cursorx,cursory-3, cursorx,cursory+8, cursorx+4,cursory+8, cursorx+4,cursory-3)
  'Now, update cursor position and draw the new cursor
  cursorx := NewPosX  
  cursory := NewPosY  
  gr.color(2)
  gr.quad(cursorx,cursory, cursorx,cursory+8, cursorx+4,cursory+8, cursorx+4,cursory)
  return
' DrawCursor  

''---------------------------------------------------------------------
''ScrollTextUp
''Purpose: Clears the text window, redraws it one line scrolled up
''Params: none
''Returns:nothing
''---------------------------------------------------------------------
PRI ScrollTextUp | looprow, loopcol
' First, clear it all
  gr.color(0)
  gr.quad(-SCREEN_WIDTH/2,-TEXT_START_Y+TEXTLINEHEIGHT+3, SCREEN_WIDTH/2,-TEXT_START_Y+TEXTLINEHEIGHT+3, SCREEN_WIDTH/2,-TEXT_START_Y - 5*TEXTLINEHEIGHT, -SCREEN_WIDTH/2,-TEXT_START_Y - 5*TEXTLINEHEIGHT)
  gr.color(2)
' Now, shift the text up and redraw it
  repeat looprow from 1 to 4
    repeat loopcol from 0 to 31
      if (loopcol < linelength[looprow]+1)
        screentext[(looprow-1)*TEXTCHARS + loopcol] := screentext[looprow*TEXTCHARS + loopcol]
        sbuffer[0] := screentext[looprow*TEXTCHARS + loopcol]
        sbuffer[1] := 0
        gr.text(-SCREEN_WIDTH/2 + loopcol*6,-TEXT_START_Y - (looprow-1)*TEXTLINEHEIGHT, @sbuffer)
      else
        screentext[(looprow-1)*TEXTCHARS + loopcol] := " "
    linelength[looprow-1] := linelength[looprow]
  return
' ScrollTextUp

''---------------------------------------------------------------------
''runCommand
''Purpose: Runs a LOGO command
''Params: Command - which command to run
''        ParameterList - pointer to start of parameters
''Returns:nothing
''---------------------------------------------------------------------
PRI runCommand(Command, ParameterList) : NewStringPosition | parameter1, parameter2, oldTurtlex, oldTurtley, Loop, lineBackup, Loop2, EndReached, ParamPos, stackpos, pageoffset, FunctionPage, EndTOC, TOCaddress, testchar, commandchar
  oldTurtlex := turtlex
  oldTurtley := turtley
  lineBackup := newLine
  NewStringPosition := ParameterList

  case Command
    COMMAND_FD, COMMAND_FORWARD:
      eraseTurtle
      parameter1 := getIntParameter(ParameterList)
      ParameterList := secondaryReturn
      'standard geometry, x = delta * cos(angle), y = delta * sin(angle).  +90 so zero degrees points straight up.
      turtlex := turtlex + ((parameter1 * getCos(turtleangle+90)) / 65535)
      turtley := turtley + ((parameter1 * getSin(turtleangle+90)) / 65535)
      'plot the line
      if (pendown == 1)
        gr.colorwidth(2,0)
        gr.plot(oldTurtlex, oldTurtley)
        gr.line(turtlex, turtley)      
      drawTurtle
      NewStringPosition := ParameterList       

    COMMAND_BK, COMMAND_BACKWARD:
      eraseTurtle
      parameter1 := getIntParameter(ParameterList)
      ParameterList := secondaryReturn
      'standard geometry, x = delta * cos(angle), y = delta * sin(angle).  +90 so zero degrees points straight up.
      turtlex := turtlex - ((parameter1 * getCos(turtleangle+90)) / 65535)
      turtley := turtley - ((parameter1 * getSin(turtleangle+90)) / 65535)
      'plot the line
      if (pendown == 1)
        gr.colorwidth(2,0)
        gr.plot(oldTurtlex, oldTurtley)
        gr.line(turtlex, turtley)      
      drawTurtle
      NewStringPosition := ParameterList       

    COMMAND_LT, COMMAND_LEFT:
      eraseTurtle
      parameter1 := getIntParameter(ParameterList)
      ParameterList := secondaryReturn
      turtleangle += parameter1
      if (turtleangle => 360)
        turtleangle -= 360
      drawTurtle
      NewStringPosition := ParameterList       
    
    COMMAND_RT, COMMAND_RIGHT:
      eraseTurtle
      parameter1 := getIntParameter(ParameterList)
      ParameterList := secondaryReturn
      turtleangle := turtleangle - parameter1
      if (turtleangle < 0)
        turtleangle += 360
      drawTurtle
      NewStringPosition := ParameterList       

    COMMAND_HOME:      ' Warp turtle back to 0,0 (draw a line, though, if pendown)
      eraseTurtle
      turtlex := 0
      turtley := 0
      'plot the line
      if (pendown == 1)
        gr.colorwidth(2,0)
        gr.plot(oldTurtlex, oldTurtley)
        gr.line(turtlex, turtley)      
      drawTurtle
      NewStringPosition := ParameterList - 1       

    COMMAND_SETPOS:    ' Warp turtle
      eraseTurtle
      parameter1 := getIntParameter(ParameterList)
      turtlex := parameter1
      ParameterList := secondaryReturn + 1
      parameter1 := getIntParameter(ParameterList)
      turtley := parameter1
      ParameterList := secondaryReturn
      drawTurtle
      NewStringPosition := ParameterList       

    COMMAND_HIDETURTLE, COMMAND_HT:' Invisible turtle
      eraseTurtle
      turtlevisible := 0
      NewStringPosition := ParameterList - 1       

    COMMAND_SHOWTURTLE, COMMAND_ST:' Visible turtle
      turtlevisible := 1
      drawTurtle
      NewStringPosition := ParameterList - 1       

    COMMAND_PENUP, COMMAND_PU:     ' Stop drawing
      pendown := 0
      NewStringPosition := ParameterList - 1
      
    COMMAND_PENDOWN,COMMAND_PD:    ' Start drawing
      pendown := 1
      NewStringPosition := ParameterList - 1       
   
    COMMAND_REPEAT, COMMAND_RP:
      parameter1 := getIntParameter(ParameterList)      ' times to repeat
      ParameterList := secondaryReturn
      NewStringPosition := ParameterList                  ' char position following parameter we just read

      repeat Loop from 1 to parameter1
        newLine := lineBackup                                        ' interpretCommand resets newLine
        runBracketedCommands(NewStringPosition)
'        Print_To_Term(screentext[NewStringPosition])
        
        if (EscapeHit == 1)
          return
          
      newLine := lineBackup                                        ' interpretCommand resets newLine
 
      if (genericError == ERROR_NORMAL)
        genericError := ERROR_REPEAT
  
    COMMAND_TO:        ' create function at end of command list
      parameter1 := getNewCommandName(ParameterList)
      ParameterList := secondaryReturn
      
      Loop2 := COMMAND_LENGTH + COMMAND_MAX + 1    ' Get end of command list
      commands[Loop2] := parameter1                ' store length of new command name

      repeat Loop from 0 to (parameter1 - 1)       ' store newcommand name
        Loop2++ 
        commands[Loop2] := newcommand[Loop]

      COMMAND_MAX++                                ' Increase number, length of commands in commandlist
      COMMAND_LENGTH += parameter1
      backupcommandlength := parameter1
      
    ' Store parameters
      functionLine[0] := 0
      Loop := 2                 ' Leave 2 bytes at the beginning, in case we want them later for something
      if ((screentext[ParameterList] <> KEYCODE_ENTER) and (screentext[ParameterList+1] == ":"))
        ParameterList++
        repeat while (screentext[ParameterList] <> KEYCODE_ENTER)
          newfunction[Loop] := screentext[ParameterList]
          Loop++
          ParameterList++

      newfunction[Loop] := KEYCODE_ENTER 

      functionLine[0]++
      toMode := 1                                  ' Turn on "to" mode, store commands rather than run them
      functionPos[0] := 0
      NewStringPosition := ParameterList

    COMMAND_MAKE,  COMMAND_MK:     ' set variable, do equations
      NewStringPosition := ParameterList - 1       

    COMMAND_SHOOT:     ' Bang!
      NewStringPosition := ParameterList - 1       

    COMMAND_RESET, COMMAND_NOOVERSCAN, COMMAND_OVERSCAN:     ' Clear screen, reset Logo
    ' init tile screen, turtle-screen backup
      gr.clear
      repeat LOOP from 1 to 32
        screenbackup[LOOP] := 0
        screenbackup[32+LOOP] := 0
        screenbackup[64+LOOP] := 0

      if (command == COMMAND_NOOVERSCAN)
        TITLE_Y := TITLE_Y_NOOVERSCAN
      elseif (command == COMMAND_OVERSCAN) 
        TITLE_Y := TITLE_Y_OVERSCAN
      TEXT_START_Y := TITLE_Y + 16
      
      ' write title
      gr.textmode(1,1,5,0)
      gr.colorwidth(2,0)
      gr.text(-SCREEN_WIDTH/2, -TITLE_Y, @title_string)
      gr.colorwidth(3,0)
      gr.plot(-192/2, -TITLE_Y)
      gr.line(192/2,  -TITLE_Y)

      'draw the turle, text cursor
      turtlex := 0
      turtley := 30
      turtleangle := 0
      gr.colorwidth(1,0)
      gr.vec(turtlex, turtley, $100, turtleangle, @turtlesprite)
      column :=0
      row := 0
      DrawCursor(-SCREEN_WIDTH/2 + column*6, -TEXT_START_Y - row*TEXTLINEHEIGHT + 4)

      newLine := NO_NEW_LINE
      commandPos := 0
      commandStackPos := 0
      turtlevisible := 1
      pendown := 1
      toMode := 0
      NewStringPosition := ParameterList       

    COMMAND_PLAYSONG:      ' Plays a song in the song data table, starting at byte passed in
      songPos := getIntParameter(ParameterList)
      NewStringPosition := secondaryReturn 
      NewStringPosition := ParameterList       

    COMMAND_SAVE:          ' Saves command TOC (table of contents) to EEPROM
      EndTOC := @commands + INTERNAL_COMMAND_LENGTH + COMMAND_END + 1 + ENDTOC_START
      byte[EndTOC+0] := COMMAND_LENGTH >> 8
      byte[EndTOC+1] := COMMAND_LENGTH & $FF
      byte[EndTOC+2] := COMMAND_MAX >> 8
      byte[EndTOC+3] := COMMAND_MAX & $FF
      byte[EndTOC+4] := $FE     ' Checksum
      byte[EndTOC+5] := $ED
      eeprom.WritePages(@commands + INTERNAL_COMMAND_LENGTH + COMMAND_END + 1, COMMAND_TOC_PAGE, 1)
      Print_Message(@savedmessage)
      NewStringPosition := ParameterList - 1       

    COMMAND_CLEARSAVE:         ' Remove all user-created commands from RAM and EEPROM
      bytefill(@commands + INTERNAL_COMMAND_LENGTH + COMMAND_END + 1, 0, 256)
      eeprom.WritePages(@commands + INTERNAL_COMMAND_LENGTH + COMMAND_END + 1, COMMAND_TOC_PAGE, 1)
      COMMAND_MAX := COMMAND_END
      COMMAND_LENGTH := INTERNAL_COMMAND_LENGTH

      NewStringPosition := ParameterList - 1       

    COMMAND_IF:
      parameter1 := getIntParameter(ParameterList)
      ParameterList := secondaryReturn + 1
      testchar := screentext[ParameterList]
      ParameterList += 2
      parameter2 := getIntParameter(ParameterList)
      ParameterList := secondaryReturn

      ifresult[commandStackPos] := 0

      if ((testchar == "=") and (parameter1 == parameter2)) 
        ifresult[commandStackPos] := 1
      elseif ((testchar == "<") and (parameter1 < parameter2)) 
        ifresult[commandStackPos] := 1
      elseif ((testchar == ">") and (parameter1 > parameter2)) 
        ifresult[commandStackPos] := 1
      elseif ((testchar == "!") and (parameter1 <> parameter2))
        ifresult[commandStackPos] := 1

      ' Run the commands if true if a bracket exists
      NewStringPosition := ParameterList + 1
      if ((screentext[NewStringPosition] == "[") and (ifresult[commandStackPos] == 1))
        NewStringPosition := runBracketedCommands(NewStringPosition) 
        if (EscapeHit == 1)
          return

    COMMAND_BLOCKOPEN:
      ifblock[commandStackPos] := 1
      NewStringPosition := ParameterList - 1       

    COMMAND_BLOCKCLOSE:
      ifblock[commandStackPos] := 0
      NewStringPosition := ParameterList - 1       

    COMMAND_PRINT:          ' Print a line of text to the scren
    ' Backup this text line, as we are about to overwrite it with text
      repeat Loop from 0 to TEXTCHARS 
        textbackup[Loop] := screentext[newLine*TEXTCHARS + Loop]
    
      loop := 0
      if (screentext[loop+ParameterList] == "[")
        ParameterList++
        repeat while ((screentext[loop+ParameterList] <> "]") and (screentext[loop+ParameterList] <> KEYCODE_ENTER))
          userMessage[loop] := screentext[loop+ParameterList]
          loop++
      else
        repeat while ((screentext[loop+ParameterList] <> KEYCODE_ENTER) and (screentext[loop+ParameterList] <> "]")) 
          userMessage[loop] := screentext[loop+ParameterList]
          loop++
      userMessage[loop] := 0
      NewStringPosition := loop + ParameterList - (newLine*TEXTCHARS)
      Print_Message(@userMessage)

    ' Restore backed-up command line
      newLine := row
      repeat Loop from 0 to TEXTCHARS 
        screentext[newLine*TEXTCHARS + Loop] := textbackup[Loop]
      NewStringPosition += newLine*TEXTCHARS + 1

    COMMAND_STOP:
      endOfFunction[commandStackPos] := 1
      NewStringPosition := ParameterList - 1       

    COMMAND_SETTURBO:
      commandchar := 0
      repeat Loop2 from 0 to COMMAND_MAX
        testchar := commandchar+1
        Loop := 0
        repeat while(screentext[ParameterList + Loop] == commands[testchar])
        ' Check all letters of this command with the entered text
          testchar++
          Loop++

        if ((Loop == commands[commandchar]) and (screentext[ParameterList + Loop] == KEYCODE_ENTER))
          'We found one! Set it to turbo and exit
          ParameterList += Loop
          if (commandStackPos == 0)              
            repeat Loop from 0 to commands[commandchar]-1
              userMessage[loop] := commands[commandchar + Loop + 1]
            userMessage[loop] := "!"  
            userMessage[loop+1] := 0
            Print_Message(@userMessage)

          turbo_page := ((Loop2-COMMAND_END-1)*PAGES_PER_FUNCTION) + FUNCTION_START_PAGE
          eeprom.SetTurboPage(turbo_page,@newfunction)
          return(ParameterList)
        else
        ' Advance to next command
          commandchar := commandchar + commands[commandchar] + 1
      
      NewStringPosition := ParameterList - 1       

    COMMAND_IFPAD1, COMMAND_IFPAD2:
      parameter1 := getPadParameter(ParameterList)
      ParameterList := secondaryReturn

      if (command == COMMAND_IFPAD2)
        parameter1 <<= 8        ' second pad uses high 8-bits 
      
      if (controller.button(parameter1)) 
        ifresult[commandStackPos] := 1
      else
        ifresult[commandStackPos] := 0

      ' Run the commands if true
      NewStringPosition := ParameterList + 1
      if ((screentext[NewStringPosition] == "[") and (ifresult[commandStackPos] == 1))
        NewStringPosition := runBracketedCommands(NewStringPosition) 
        if (EscapeHit == 1)
          return
        
      NewStringPosition := ParameterList       
      
    COMMAND_END:
      NewStringPosition := ParameterList       

    OTHER:             ' User-made command
      if (commandStackPos + 1 == COMMAND_STACKSIZE)     ' Don't exceed the stacksize
        return (NewStringPosition)

      FunctionPage := ((command-COMMAND_END-1)*PAGES_PER_FUNCTION) + FUNCTION_START_PAGE
    ' First, backup the current line we are running
      funcBackupCommandLine(ParameterList)

      commandStackPos++

    ' Reset command-specific variables
      endOfFunction[commandStackPos] := 0
      ifblock[commandStackPos] := 0
      ifresult[commandStackPos] := 0

      ' Load up on local variables to be passed in
      ParameterList := funcPrepareLocalVariables(FunctionPage, ParameterList)
      NewStringPosition := ParameterList - 1
 
    ' Now run the function
      functionLine[commandStackPos]++
      eeprom.ReadLines(@newfunction, FunctionPage, functionLine[commandStackPos], @EeepromBuffer, 1)
      commandFinalLine[commandStackPos]--
            
      EndReached := 0
      repeat while (EndReached == 0) 
      ' Copy each line of the function to the current internal text line and interpret it
        Loop := funcCopyBufferToText
          
        newLine := row
        commandPos := 0
        linelength[row] := Loop

        if ((ifblock[commandStackPos] == 0) or (ifresult[commandStackPos] == 1) or (screentext[newLine*TEXTCHARS] == "}"))
          if (commandFinalLine[commandStackPos] - 1 == 0)
            endOfFunction[commandStackPos] := 1  
          Loop := interpretCommand

'-----------------------------------------------------
        ' This was the end of the function - and calling a new one.  Just do it now, to save on stack space.          
          if (endOfFunction[commandStackPos] > 1)   ' interpretCommand changes this to the new command    
            command := endOfFunction[commandStackPos]
            endOfFunction[commandStackPos] := 0
            ParameterList := Loop
            lineBackup := newLine
            FunctionPage := ((command-COMMAND_END-1)*PAGES_PER_FUNCTION) + FUNCTION_START_PAGE

          ' Reset command-specific variables
            ifblock[commandStackPos] := 0
            ifresult[commandStackPos] := 0

            ' Load up on local variables to be passed in
            ParameterList := funcPrepareLocalVariables(FunctionPage, ParameterList)
            NewStringPosition := ParameterList - 1
 
          ' Now run the function
            functionLine[commandStackPos]++
            eeprom.ReadLines(@newfunction, FunctionPage, functionLine[commandStackPos], @EeepromBuffer, 1)
            commandFinalLine[commandStackPos]--
            
            EndReached := 0

          ' Copy each line of the function to the current internal text line and interpret it
            Loop := funcCopyBufferToText
          
            newLine := row
            commandPos := 0
            linelength[row] := Loop       

            interpretCommand
            if (commandFinalLine[commandStackPos] - 1 == 0)
              endOfFunction[commandStackPos] := 1  
          elseif (commandFinalLine[commandStackPos] - 1 == 0)
            endOfFunction[commandStackPos] := 1  
  
'-----------------------------------------------------
        if (EscapeHit == 1)
          return

        if (endOfFunction[commandStackPos] == 0)  
          functionLine[commandStackPos]++
          pageoffset := functionLine[commandStackPos] / 8  
          eeprom.ReadLines(@newfunction, FunctionPage + pageoffset, functionLine[commandStackPos]-(pageoffset*8), @EeepromBuffer, 1)
          commandFinalLine[commandStackPos]--
        else
          EndReached := 1
          endOfFunction[commandStackPos] := 0

      commandStackPos--

    ' Restore the original text line
      ParameterList := funcRestoreCommandLine
'      NewStringPosition := ParameterList - 1

  return (NewStringPosition)
' runCommand

''---------------------------------------------------------------------
''funcBackupCommandLine
''Purpose: Backup the current command-line and whatnot
''Params: ParameterList - pointer to start of parameters
''Returns:None
''---------------------------------------------------------------------
PRI funcBackupCommandLine(ParameterList) | Loop
  functionBackupCommandPos[commandStackPos] := commandPos
  functionBackupLinelength[commandStackPos] := linelength[row]
  functionParameterList[commandStackPos] := ParameterList
  repeat Loop from 0 to linelength[row]+1
    functionBackupLine[commandStackPos*TEXTCHARS + Loop] := screentext[row*TEXTCHARS + Loop]
' funcBackupCommandLine

''---------------------------------------------------------------------
''funcRestoreCommandLine
''Purpose: Restore the current command-line and whatnot
''Params: None
''Returns:ParameterList - pointer to start of parameters for current line
''---------------------------------------------------------------------
PRI funcRestoreCommandLine | Loop
  commandPos := functionBackupCommandPos[commandStackPos]
  linelength[row] := functionBackupLinelength[commandStackPos]
  repeat Loop from 0 to linelength[row]+1
    screentext[row*TEXTCHARS + Loop] := functionBackupLine[commandStackPos*TEXTCHARS + Loop]
  return (functionParameterList[commandStackPos])
' funcRestoreCommandLine

''---------------------------------------------------------------------
''funcPrepareLocalVariables
''Purpose: Setup holders for local function variables (ie, :size :angle)
''Params: FunctionPage  - page this function is on
''Returns:ParameterList - pointer to start of parameters for current line
''---------------------------------------------------------------------
PRI funcPrepareLocalVariables(FunctionPage, ParameterList) | Loop, Loop2, stackpos, ParamPos
  functionLine[commandStackPos] := 0
  eeprom.ReadLines(@newfunction, FunctionPage, functionLine[commandStackPos], @EeepromBuffer, 1)
  commandFinalLine[commandStackPos] := EeepromBuffer[0] - 1 

  stackpos := commandStackPos*VARIABLES_LOCALMAX*VARIABLE_MAXCHARS
  ParamPos := 2
  if (EeepromBuffer[ParamPos] == ":") 
    Loop := 0
    newfunctionLocalVarNum[commandStackPos] := 0
    repeat while (EeepromBuffer[ParamPos] == ":")
      Loop2 := 0
      ParamPos++
      repeat
        newfunctionlocalvars[stackpos + Loop] := EeepromBuffer[ParamPos]
        ParamPos++
        Loop++
        Loop2++
      while ((EeepromBuffer[ParamPos] <> " ") and (EeepromBuffer[ParamPos] <> KEYCODE_ENTER) and (Loop2 < VARIABLE_MAXCHARS))
      ParamPos++
      newfunctionLocalVarNum[commandStackPos]++
      repeat while (Loop2 < 8)
        newfunctionlocalvars[stackpos + Loop] := " "
        Loop++
        Loop2++

  ' Grab any passed-in variables
  if (newfunctionLocalVarNum[commandStackPos] > 0)
    repeat Loop from 1 to newfunctionLocalVarNum[commandStackPos]
      localVariableStorage[commandStackPos*VARIABLES_LOCALMAX + Loop] := getIntParameter(ParameterList)
        ParameterList := secondaryReturn + 1

  return(ParameterList)                  
' funcPrepareLocalVariables

''---------------------------------------------------------------------
''funcCopyBufferToText
''Purpose: Setup holders for local function variables (ie, :size :angle)
''Params: FunctionPage  - page this function is on
''Returns:ParameterList - pointer to start of parameters for current line
''---------------------------------------------------------------------
PRI funcCopyBufferToText | Loop, Loop2
  Loop := 0
  Loop2 := 0      
  repeat while (EeepromBuffer[Loop] <> KEYCODE_ENTER)
    if (EeepromBuffer[Loop] == ":")  ' Replace Variable name with value
      Loop := SetLocalVariableValue(Loop+1, Loop2)
      Loop2 := secondaryReturn
    else 
      screentext[row*TEXTCHARS + Loop2] := EeepromBuffer[Loop]
      Loop++
      Loop2++
  screentext[row*TEXTCHARS + Loop2] := KEYCODE_ENTER

  return(Loop)
' funcCopyBufferToText        

''---------------------------------------------------------------------
''runBracketedCommands
''Purpose: Run a list of commands inside of brackets on a single line
''Params: NewStringPosition - where on the current line to start reading
''Returns:None
''---------------------------------------------------------------------
PRI runBracketedCommands(NewStringPosition) | lineBackup, stringposBackup
  lineBackup := newLine                                   

  repeat while (screentext[NewStringPosition] <> "]")
    newLine := lineBackup                                        ' interpretCommand resets newLine
    commandPos := (NewStringPosition+1) - (newLine * TEXTCHARS)  'NewStringPosition is the position in 5-line string,
    NewStringPosition := interpretCommand                        'I need the position in the current line only
    if (EscapeHit == 1)
      return (NewStringPosition)

  return (NewStringPosition)
' runBracketedCommands
 
''---------------------------------------------------------------------
''SetLocalVariableValue
''Purpose: Gets an integer parameter from the passed-in list
''Params: ParameterList - pointer to start of parameters
''Returns:parameter (long)
''---------------------------------------------------------------------
PRI SetLocalVariableValue(CurFunctionPos, CurScreenPos) | WhichVariable,Loop,Loop2,Value,ValueBak, stackpos
  ' First, figure out which variable the command wants to use
  stackpos := commandStackPos*VARIABLES_LOCALMAX*VARIABLE_MAXCHARS 
  WhichVariable := -1
  repeat
    WhichVariable++
    Loop := CurFunctionPos
    Loop2 := 0
    repeat while ((EeepromBuffer[Loop] == newfunctionlocalvars[stackpos + ((WhichVariable)*VARIABLE_MAXCHARS) + Loop2]) and (EeepromBuffer[Loop] <> " ") and (EeepromBuffer[Loop] <> KEYCODE_ENTER))
      Loop++
      Loop2++
  while ((newfunctionlocalvars[stackpos + ((WhichVariable)*VARIABLE_MAXCHARS) + Loop2] <> " ") and (newfunctionlocalvars[stackpos + ((WhichVariable)*VARIABLE_MAXCHARS) + Loop2] <> KEYCODE_ENTER))  

'  Print_To_Term(localVariableStorage[commandStackPos*VARIABLES_LOCALMAX + 1]/10 + "0")

  WhichVariable++
  Value := localVariableStorage[commandStackPos*VARIABLES_LOCALMAX + WhichVariable]
  ValueBak := Value
 
  ' OK, gottit.  Now write the contents of that variable to the screen
  if (Value < 0)
    screentext[row*TEXTCHARS + CurScreenPos] := "-"
    Value *= -1
    CurScreenPos++

  if (Value > 99)
    screentext[row*TEXTCHARS + CurScreenPos] := (Value / 100) + "0"
    Value -= ((Value / 100)*100)
    CurScreenPos++
    
  if (Value > 9)
    screentext[row*TEXTCHARS + CurScreenPos] := (Value / 10) + "0"
    Value -= ((Value / 10)*10)
    CurScreenPos++
  elseif (ValueBak > 99)        ' Make sure we draw the second 0
    screentext[row*TEXTCHARS + CurScreenPos] := "0"
    CurScreenPos++
  else 
'    Print_To_Term("!")
    
  screentext[row*TEXTCHARS + CurScreenPos] := Value + "0"
  CurScreenPos++

  secondaryReturn := CurScreenPos
  return (Loop)
' SetLocalVariableValue

''---------------------------------------------------------------------
''getIntParameter
''Purpose: Gets an integer parameter from the passed-in list
''Params: ParameterList - pointer to start of parameters
''Returns:parameter (long)
''---------------------------------------------------------------------
PRI getIntParameter(ParameterList) | Parameter, Loop, Multiplier, SecondValue, Operation, FinalValue
  Parameter := 0
  
  ' Get the length (in chars) of the parameter    
  Loop := 0
  repeat while((screentext[ParameterList + Loop] <> " ") and (screentext[ParameterList + Loop] <> "[") and (screentext[ParameterList + Loop] <> "]") and (screentext[ParameterList + Loop] <> KEYCODE_ENTER))
    Loop++

  secondaryReturn := ParameterList + Loop

  Operation := 0
  
  ' Change the ascii parameter into an integer number
  Parameter := Parameter + (screentext[ParameterList+Loop-1] - "0")
  Multiplier := 0
  Loop--   
  repeat while (Loop > 0)
    ' Check for valid parameter
    if ((Loop == 1) and (screentext[ParameterList+Loop-1] == "-"))
      Parameter *= -1
    elseif (screentext[ParameterList+Loop-1] == "-")
      Operation := "-"
      Multiplier := 0
      Loop--
      SecondValue := screentext[ParameterList+Loop-1] - "0"
    elseif (screentext[ParameterList+Loop-1] == "+")
      Operation := "+"
      Multiplier := 0
      Loop--
      SecondValue := screentext[ParameterList+Loop-1] - "0"
    elseif (screentext[ParameterList+Loop-1] == "*")
      Operation := "*"
      Multiplier := 0
      Loop--
      SecondValue := screentext[ParameterList+Loop-1] - "0"
    elseif (screentext[ParameterList+Loop-1] == "/")
      Operation := "/"
      Multiplier := 0
      Loop--
      SecondValue := screentext[ParameterList+Loop-1] - "0"
    elseif ((screentext[ParameterList+Loop-1] < "0") or (screentext[ParameterList+Loop-1] > "9"))
      return(0)
    else
        if (Operation == 0)
          Parameter := Parameter + ((screentext[ParameterList+Loop-1] - "0") * powerOfTen(Multiplier))
        else
          SecondValue := SecondValue + ((screentext[ParameterList+Loop-1] - "0") * powerOfTen(Multiplier))
        Multiplier++
    Loop--

  if (Operation <> 0)
    if (Operation == "-")
      FinalValue := (SecondValue - Parameter)
    elseif (Operation == "+")
      FinalValue := (SecondValue + Parameter)
    elseif (Operation == "*")
      FinalValue := (SecondValue * Parameter)
    elseif (Operation == "/")
      FinalValue := (SecondValue / Parameter)
    Parameter := FinalValue

  return(Parameter)
  ' getIntParameter

''---------------------------------------------------------------------
''getPadParameter
''Purpose: Gets an integer parameter from the passed-in list
''Params: ParameterList - pointer to start of parameters
''Returns:parameter (long)
''---------------------------------------------------------------------
PRI getPadParameter(ParameterList)

  if (screentext[ParameterList] == "r")
    secondaryReturn := ParameterList+5
    return (NES0_RIGHT) 
  elseif (screentext[ParameterList] == "l")
    secondaryReturn := ParameterList+4
    return (NES0_LEFT) 
  elseif (screentext[ParameterList] == "d")
    secondaryReturn := ParameterList+4
    return (NES0_DOWN) 
  elseif (screentext[ParameterList] == "u")
    secondaryReturn := ParameterList+2
    return (NES0_UP) 
  elseif (screentext[ParameterList] == "b")
    secondaryReturn := ParameterList+1
    return (NES0_B) 
  elseif (screentext[ParameterList] == "a")
    secondaryReturn := ParameterList+1
    return (NES0_A)
  elseif (screentext[ParameterList] == "s")
    if (screentext[ParameterList+1] == "t")
      secondaryReturn := ParameterList+5
      return (NES0_START)
    elseif (screentext[ParameterList+1] == "e")
      secondaryReturn := ParameterList+6
      return (NES0_SELECT)
  
  return($FF)
' getPadParameter

''---------------------------------------------------------------------
''getNewCommandName
''Purpose: Gets the name of the new command from the passed-in list
''Params: ParameterList - pointer to start of parameters
''Returns:Number of characters in string
''---------------------------------------------------------------------
PRI getNewCommandName(ParameterList) | Loop
  ' Get the length (in chars) of the parameter    
  Loop := 0
  repeat while((screentext[ParameterList + Loop] <> " ") and (screentext[ParameterList + Loop] <> KEYCODE_ENTER))
    newcommand[Loop] := screentext[ParameterList + Loop] 
    Loop++

  secondaryReturn := ParameterList + Loop
  return(Loop)    

''---------------------------------------------------------------------
''powerOfTen
''Purpose: Performs mathematical "power of" by value 10
''Params: Multiplier
''Returns:result (long)
''---------------------------------------------------------------------
PRI powerOfTen(Multiplier) | Loop, Number
  Number := 1
  repeat Loop from 0 to Multiplier
    Number *= 10    
  return(Number)
' powerOfTen  
  
''---------------------------------------------------------------------
''eraseTurtle
''Purpose: Erase the turtle
''Params: nothing
''        nothing
''Returns:nothing
''---------------------------------------------------------------------
PRI eraseTurtle | looprow,loopcol
  if (turtlevisible == 1)
    gr.colorwidth(0,0)
    gr.vec(turtlex, turtley, $100, convertDegrees(turtleangle), @turtlesprite)              ' Draw Turtle
    restoreRect(turtlex, turtley)
  return
' eraseTurtle

''---------------------------------------------------------------------
''drawTurtle
''Purpose: Draw the turtle
''Params: nothing
''        nothing
''Returns:nothing
''---------------------------------------------------------------------
PRI drawTurtle | looprow,loopcol
  if (turtlevisible == 1)
    backupRect(turtlex, turtley)
    gr.colorwidth(1,0)
    gr.vec(turtlex, turtley, $100, convertDegrees(turtleangle), @turtlesprite)              ' Draw Turtle
  return
' drawTurtle

''---------------------------------------------------------------------
''convertDegrees
''Purpose: Convert degrees to the gr.vec degree format as accurately as possible
''Params: Degree value
''Returns:gr.vec degree format (0...8192 = 0..360 degrees)
''---------------------------------------------------------------------
PRI convertDegrees(Degree) : GrVecDegree
  GrVecDegree := ( (Degree * 22) + ((Degree*3)/4) )
  return (GrVecDegree)
' convertDegrees

''---------------------------------------------------------------------
''getSin
''Purpose: Get a sine value from the table
''Params: Index value in degrees
''Returns:Sine value for Index
''---------------------------------------------------------------------
PRI getSin(Index) : Sine | angle
  angle := convertDegrees(Index)

  Sine := angle << 1 & $FFE
  if angle & NATIVE_ANGLE_90
     Sine := word[SIN_TABLE_END - Sine]
  else
     Sine := word[SIN_TABLE + Sine]
  if angle & NATIVE_ANGLE_180
     -Sine

  return (Sine)
' getSin

''---------------------------------------------------------------------
''getCos
''Purpose: Get a cosine value from the table
''Params: Index value in degrees
''Returns:Cos value for Index
''---------------------------------------------------------------------
PRI getCos(Index) : Cosine
  Index += 90
  Cosine := getSin(Index)

  return (Cosine)
' getCos

''---------------------------------------------------------------------
''backupRect
''Purpose: Backup a part of the screen graphics
''Params: X,Y           Starting pixel pos
''Returns:Position in Screen coordinates
''---------------------------------------------------------------------
PRI backupRect(X,Y) | looprow
  X := mapToScreen(X, TRUE) >> 4
  Y := mapToScreen(Y, FALSE) 

  repeat looprow from 1 to 32
    screenbackup[looprow] := GetBasicTile(X-1,Y-12+looprow)
    screenbackup[32+looprow] := GetBasicTile(X,Y-12+looprow)
    screenbackup[64+looprow] := GetBasicTile(X+1,Y-12+looprow)
' backupRect 

''---------------------------------------------------------------------
''restoreRect
''Purpose: Restore a part of the screen graphics from the backup
''Params: X,Y           Starting pixel pos
''Returns:Position in Screen coordinates
''---------------------------------------------------------------------
PRI restoreRect(X,Y) | looprow,loopcol
  X := mapToScreen(X, TRUE) >> 4
  Y := mapToScreen(Y, FALSE)

  repeat looprow from 1 to 32
    SetBasicTile(X-1,Y-12+looprow, screenbackup[looprow])
    SetBasicTile(X,Y-12+looprow, screenbackup[32+looprow])
    SetBasicTile(X+1,Y-12+looprow, screenbackup[64+looprow])
' restoreRect 

''---------------------------------------------------------------------
''mapToScreen
''Purpose: Convert map coordinates to screen coordinates
''Params: MapPos  Position in Map
''Returns:Position in Screen coordinates
''---------------------------------------------------------------------
PRI mapToScreen(MapPos, IsX) : ScreenPos
  if (IsX)
    ScreenPos := MapPos + SCREEN_WIDTH >> 1
  else
    ScreenPos := LONGSPERCOL >> 1 - MapPos       
  return (ScreenPos)
' mapToScreen

''---------------------------------------------------------------------
''SetBasicTile
''Purpose: Set the contents of a tile on the screen
''Params: MapX,MapY       Position on screen
''        Colorvalues     Contents to place there (16 pixels, 4 pixels per byte, 2 bits per pixel)
''Returns:Nothing
''---------------------------------------------------------------------
PRI SetBasicTile(MapX,MapY, Colorvalues) | Offset
  Offset := LONGBYTES*(LONGSPERCOL*MapX+MapY) 
  if ((Offset => 0) and (Offset < SCREEN_BUFFER_LONGS*4))
    LONG[ONSCREEN_BUFFER + Offset] := Colorvalues
  return
' SetBasicTile

''---------------------------------------------------------------------
''GetBasicTile
''Purpose: Set the contents of a tile on the screen
''Params: MapX,MapY       Position on screen
''        Colorvalues     Contents to place there (16 pixels, 4 pixels per byte, 2 bits per pixel)
''Returns:Nothing
''---------------------------------------------------------------------
PRI GetBasicTile(MapX,MapY) : Colorvalues
  Colorvalues := LONG[ONSCREEN_BUFFER + (LONGBYTES*(LONGSPERCOL*MapX+MapY))]
  return (Colorvalues)
' GetBasicTile

''---------------------------------------------------------------------
''LoadFromEeprom
''Purpose: Loadup user-created commands from the EEPROM, if they exist
''Params: None
''Returns:Nothing
''---------------------------------------------------------------------
PRI LoadFromEeprom | TOCaddress
  TOCaddress := @commands + INTERNAL_COMMAND_LENGTH + COMMAND_END + 1 
  eeprom.ReadLines(0, COMMAND_TOC_PAGE, 0, TOCaddress, 8)
  if ((byte[TOCaddress+ENDTOC_START+4] == $FE) and (byte[TOCaddress+ENDTOC_START+5] == $ED))
  ' ' Good checksum means good save exists - load it!
'    Print_Message(@loadedmessage)
    COMMAND_LENGTH := (byte[TOCaddress+ENDTOC_START+0] << 8) | (byte[TOCaddress+ENDTOC_START+1])    
    COMMAND_MAX := (byte[TOCaddress+ENDTOC_START+2] << 8) | (byte[TOCaddress+ENDTOC_START+3])
    ' setup turbo for the first function
    eeprom.ReadLines(0, FUNCTION_START_PAGE,   0, @newfunction, 8)
    eeprom.ReadLines(0, FUNCTION_START_PAGE+1, 0, @newfunction+256, 8)
    return(TRUE)
  else
    return(FALSE)
' LoadFromEeprom

'///////////////////////////////////////////////////////////////////////
' DATA SECTION /////////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

DAT

' TV PARAMETERS FOR DRIVER /////////////////////////////////////////////

tvparams
                long    0               'status
                long    1               'enable
                long    %011_0000       'pins
                long    %0000           'mode
                long    0               'screen
                long    0               'colors
                long    x_tiles         'hc
                long    y_tiles         'vc
                long    10              'hx timing stretch
                long    1               'vx
                long    0               'ho
                long    0               'vo
                long    55_250_000      'broadcast
                long    0               'auralcog

' STRING STORAGE //////////////////////////////////////////////////////

title_string    byte    "Josh Meeds' Hydra Logo v1.41",0         'text
blank_string    byte    "           ",0
huhmessage      byte    "Huh? I don't understand.",0
huhrepeatmessage byte   "That repeat had problems.",0
loadedmessage   byte    "Loaded.",0
savedmessage    byte    "Saved.",0
cancelledmessage byte   "Canceled.",0
notloadedmessage byte   "Checksum wrong.",0
toomanylinesmessage byte "Too many lines in command",0

turtlesprite            word    $4000+2048            ' 90 degrees (straight up)
                        word    10
                        word    $8000+4096            ' 180 degrees (left)
                        word    3
                        word    $8000+2048            ' 90 degrees (straight up)
                        word    3
                        word    $8000                 ' 0 degrees (right)
                        word    3
                        word    $8000+2048            ' 90 degrees (straight up)
                        word    10
                        word    0

songs                   word    NOTES_B2,1
                        word    NOTES_A2,1
                        word    NOTES_B2,4
                        word    NOTES_E2,4
                        word    NOTES_B2,8
                        word    NOTES_C3,1
                        word    NOTES_B2,1
                        word    NOTES_C3,2
                        word    NOTES_B2,2
                        word    NOTES_A2,2
                        word    NOTES_REST,0
                        
' LOGO Commands    #chars,string                                    
commands        byte    2,"fd"
                byte    7,"forward"
                byte    2,"bk"
                byte    8,"backward"
                byte    2,"lt"
                byte    4,"left"
                byte    2,"rt"
                byte    5,"right"
                byte    4,"home"
                byte    6,"setpos"
                byte    10,"hideturtle"
                byte    2,"ht"
                byte    10,"showturtle"
                byte    2,"st"
                byte    5,"penup"
                byte    2,"pu"
                byte    7,"pendown"
                byte    2,"pd"
                byte    6,"repeat"
                byte    2,"rp"
                byte    2,"to"
                byte    4,"make"
                byte    2,"mk"
                byte    5,"shoot"
                byte    5,"reset"
                byte    8,"playsong"
                byte    4,"save"
                byte    9,"clearsave"
                byte    2,"if"
                byte    1,"["
                byte    1,"]"
                byte    5,"print"
                byte    4,"stop"
                byte    8,"setturbo"
                byte    10,"nooverscan"
                byte    8,"overscan"
                byte    6,"ifpad1"
                byte    6,"ifpad2"
                byte    3,"end"
'///////////////////////////////////////////////////////////////////////
'///NOTHING BEYOND HERE - THIS IS RESERVED FOR USER-CREATED COMMANDS ///
'///////////////////////////////////////////////////////////////////////